package cloud.javastudy.study.demo.dao;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.SelectProvider;
import org.apache.ibatis.builder.annotation.ProviderMethodResolver;
import org.apache.ibatis.jdbc.SQL;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 注意两个注解, @Mapper是代表这个类是Mybatis中的Mapper
 * 还有@Repositor是代表这个类是spring的bean
 * @Author https://www.javastudy.cloud
 * @CreateTime 2019/11/4
 **/
@Mapper
@Repository
public interface TestDAO {

    /**
     * 这里简单的sql直接通过@Select注解来写了
     * 稍复杂一点的要通过SelectProvider来写
     * @return
     */
    @Select("select * from test")
    public List<Map> listTests();

    /**
     * 复杂的sql可以通过这种@SelectProvider去指定一个sql维护类
     * @param id
     * @return
     */
    @SelectProvider(type = TestProvider.class)
    public Map getById(@Param("id") Long id);

    /**
     * 注意Provider要实现ProviderMethodResolver这个接口
     * 为方便,直接使用了内部类的方式,实际开发中,一般会单独写一个类
     */
    class TestProvider implements ProviderMethodResolver {
        /**
         * sql提供类,方法名一定要和mapper里面的方法名一致
         * @param id
         * @return
         */
        public String getById(@Param("id") Long id){
            SQL sql = new SQL(){
                {
                    SELECT("*");
                    FROM("test");
                    WHERE("id = "+id);
                }
            };
            return sql.toString();
        }
    }
}